//============================================================================//
//
//  Software Name : MEGADOCK
//
//  Class Name : CPUTime
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#include "cpu_time.h"

//============================================================================//
void CPUTime::initialize()
//============================================================================//
{

    control_initialize = 0.0;
    receptor_process = 0.0;
    docking_total = 0.0;
    ligand_voxelization = 0.0;
    fftprocess_ligand_fft = 0.0;
    fftprocess_fft_inverse = 0.0;
    fftprocess_score_sort = 0.0;
    fftprocess_sort_index = 0.0;
    docking_output_detail = 0.0;
    docking_output = 0.0;


    return;
}

//============================================================================//
void CPUTime::output()
//============================================================================//
{

    const float total_time =  control_initialize+
                              receptor_process+
                              docking_total+
        					  docking_output_detail+
                              docking_output;

    if(total_time < 5.0) {
        printf("\n");
        printf("CPU time (initialize)         = %8.3f sec. (%4.1f%%)\n",control_initialize     , 100.0*control_initialize     /total_time);
        printf("CPU time (receptor process)   = %8.3f sec. (%4.1f%%)\n",receptor_process       , 100.0*receptor_process       /total_time);
        printf("CPU time (docking)            = %8.3f sec. (%4.1f%%)\n",docking_total          , 100.0*docking_total          /total_time);
        printf("  | Ligand voxelization       = %8.3f sec. (%4.1f%%)\n",ligand_voxelization    , 100.0*ligand_voxelization    /total_time);
        printf("  | Ligand FFT                = %8.3f sec. (%4.1f%%)\n",fftprocess_ligand_fft  , 100.0*fftprocess_ligand_fft  /total_time);
        printf("  | Inverse FFT to get score  = %8.3f sec. (%4.1f%%)\n",fftprocess_fft_inverse , 100.0*fftprocess_fft_inverse /total_time);
        printf("  | Sort score (each core)    = %8.3f sec. (%4.1f%%)\n",fftprocess_score_sort  , 100.0*fftprocess_score_sort  /total_time);
        printf("  | Sort score (merge)        = %8.3f sec. (%4.1f%%)\n",fftprocess_sort_index  , 100.0*fftprocess_sort_index  /total_time);
        printf("CPU time (detailed output)    = %8.3f sec. (%4.1f%%)\n",docking_output_detail  , 100.0*docking_output_detail  /total_time);
        printf("CPU time (output)             = %8.3f sec. (%4.1f%%)\n",docking_output         , 100.0*docking_output         /total_time);
    } else {
        printf("\n");
        printf("CPU time (initialize)         = %8.2f sec. (%4.1f%%)\n",control_initialize     , 100.0*control_initialize     /total_time);
        printf("CPU time (receptor process)   = %8.2f sec. (%4.1f%%)\n",receptor_process       , 100.0*receptor_process       /total_time);
        printf("CPU time (docking)            = %8.2f sec. (%4.1f%%)\n",docking_total          , 100.0*docking_total          /total_time);
        printf("  | Ligand voxelization       = %8.2f sec. (%4.1f%%)\n",ligand_voxelization    , 100.0*ligand_voxelization    /total_time);
        printf("  | Ligand FFT                = %8.2f sec. (%4.1f%%)\n",fftprocess_ligand_fft  , 100.0*fftprocess_ligand_fft  /total_time);
        printf("  | Inverse FFT to get score  = %8.2f sec. (%4.1f%%)\n",fftprocess_fft_inverse , 100.0*fftprocess_fft_inverse /total_time);
        printf("  | Sort score (each core)    = %8.2f sec. (%4.1f%%)\n",fftprocess_score_sort  , 100.0*fftprocess_score_sort  /total_time);
        printf("  | Sort score (merge)        = %8.2f sec. (%4.1f%%)\n",fftprocess_sort_index  , 100.0*fftprocess_sort_index  /total_time);
        printf("CPU time (detailed output)    = %8.2f sec. (%4.1f%%)\n",docking_output_detail  , 100.0*docking_output_detail  /total_time);
        printf("CPU time (output)             = %8.2f sec. (%4.1f%%)\n",docking_output         , 100.0*docking_output         /total_time);
    }

    return;
}
