//============================================================================//
//
//  Software Name : MEGADOCK
//
//  Class Name : FFTProcess
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#ifndef FFTProcess_h
#define FFTProcess_h 1


#include <algorithm>
#include <functional>

#include "cpu_time.h"
#include "parallel.h"
#include "parameter.h"
#include "protein.h"

#include "fftw3.h"

using namespace std;

typedef struct {
    float score;
    int   index[4];
} SortScore;

class FFTProcess
{
private:
    FFTProcess(FFTProcess &c) {}
    const FFTProcess & operator=(const FFTProcess &c);

    fftwf_complex         *_FFTWin;
    fftwf_complex         *_FFTWout;

    CPUTime           *_cputime;
    Parallel          *_parallel;
    Parameter         *_parameter;
    vector<SortScore>     _Top;
    vector< vector<SortScore> >   _Select;
    int               _Num_fft;
    float             *_FFT_rec_r;
    float             *_FFT_rec_i;
    float             **_FFT_lig_r;
    float             **_FFT_lig_i;
    int               *_Rotation;
protected:
    virtual void              alloc_fft();
    virtual void          fft3d(const float &theta,int myid2);
public:
    FFTProcess(CPUTime *pcputime,Parallel *pparallel,Parameter *pparameter)
        : _cputime(pcputime),_parallel(pparallel),_parameter(pparameter) {

    }
    virtual ~FFTProcess() {
        delete [] _Rotation;
        delete [] _FFT_rec_r;
        delete [] _FFT_rec_i;
        delete [] _FFT_lig_r;
        delete [] _FFT_lig_i;


        fftwf_free(_FFTWin);
        fftwf_free(_FFTWout);

        vector< vector<SortScore> > tmp1;
        vector<SortScore> tmp2;
        _Select.swap(tmp1);
        _Top.swap(tmp2);
    }
    virtual void      alloc_array(const int &num_fft);
    virtual void      receptor_fft(float *grid_r,float *grid_i);

    virtual void      ligand_fft(float *grid_r,float *grid_i, int myid2);
    virtual void      fft_score(int myid2);
    virtual void      score_sort(int myid2);
    virtual void      top_score_clean();
    virtual int       num_fft() {
        return _Num_fft;
    }
    virtual void      num_fft(const int &i) {
        _Num_fft = i;
    }
    virtual void      rotation_index(int i,const int &j) {
        _Rotation[i] = j;
    }
    virtual float     top_score(const int &j) {
        return _Top[j].score;
    }
    virtual int       top_index(const int &j,int k) {
        return _Top[j].index[k];
    }
    virtual void      sort_index(float *fwork,int *iwork);
};

#endif
